@@ -1,20 +1,13 @@
 use std::cmp::Ordering;
-use std::convert::Infallible;
 use std::fmt;
 use std::hash::{Hash, Hasher};
 use std::iter::{Product, Sum};
 use std::ops::*;
 use std::str::FromStr;
 
-use async_trait::async_trait;
-use bytes::Bytes;
 use collate::Collate;
-use destream::{Decoder, EncodeSeq, Encoder, FromStream, IntoStream, ToStream};
-use futures::TryFutureExt;
 use num::traits::Pow;
 use safecast::*;
-use serde::ser::{Serialize, SerializeSeq, Serializer};
-use serde::{Deserialize, Deserializer};
 
 use super::class::*;
 use super::{Error, Number, _Complex};
@@ -92,16 +85,6 @@ impl Default for Boolean {
     }
 }
 
-#[async_trait]
-impl async_hash::Hash for Boolean {
-    type Context = ();
-    type Error = Infallible;
-
-    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
-        async_hash::Hash::hash(&self.0, cxt).await
-    }
-}
-
 impl From<bool> for Boolean {
     fn from(b: bool) -> Boolean {
         Self(b)
@@ -262,45 +245,6 @@ impl fmt::Display for Boolean {
     }
 }
 
-impl Serialize for Boolean {
-    fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
-        s.serialize_bool(self.0)
-    }
-}
-
-impl<'de> Deserialize<'de> for Boolean {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        bool::deserialize(deserializer).map(Self::from)
-    }
-}
-
-#[async_trait]
-impl FromStream for Boolean {
-    type Context = ();
-
-    async fn from_stream<D: Decoder>(
-        cxt: Self::Context,
-        decoder: &mut D,
-    ) -> Result<Self, D::Error> {
-        bool::from_stream(cxt, decoder).map_ok(Self::from).await
-    }
-}
-
-impl<'en> ToStream<'en> for Boolean {
-    fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
-        e.encode_bool(self.0)
-    }
-}
-
-impl<'en> IntoStream<'en> for Boolean {
-    fn into_stream<E: Encoder<'en>>(self, e: E) -> Result<E::Ok, E::Error> {
-        e.encode_bool(self.0)
-    }
-}
-
 /// A complex number.
 #[derive(Clone, Copy)]
 pub enum Complex {
@@ -386,39 +330,26 @@ impl FloatInstance for Complex {
     }
 }
 
-#[async_trait]
-impl async_hash::Hash for Complex {
-    type Context = ();
-    type Error = Infallible;
-
-    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
-        match self {
-            Self::C32(c) => async_hash::Hash::hash(&[c.re, c.im], cxt).await,
-            Self::C64(c) => async_hash::Hash::hash(&[c.re, c.im], cxt).await,
-        }
-    }
-}
-
-impl CastFrom<[f32; 2]> for Complex {
-    fn cast_from(arr: [f32; 2]) -> Self {
+impl From<[f32; 2]> for Complex {
+    fn from(arr: [f32; 2]) -> Self {
         Self::C32(num::Complex::new(arr[0], arr[1]))
     }
 }
 
-impl CastFrom<[f64; 2]> for Complex {
-    fn cast_from(arr: [f64; 2]) -> Self {
+impl From<[f64; 2]> for Complex {
+    fn from(arr: [f64; 2]) -> Self {
         Self::C64(num::Complex::new(arr[0], arr[1]))
     }
 }
 
-impl<R, I> CastFrom<(R, I)> for Complex
+impl<R, I> From<(R, I)> for Complex
 where
-    R: CastInto<f64>,
-    I: CastInto<f64>,
+    R: Into<f64>,
+    I: Into<f64>,
 {
-    fn cast_from(value: (R, I)) -> Self {
-        let re = value.0.cast_into();
-        let im = value.1.cast_into();
+    fn from(value: (R, I)) -> Self {
+        let re = value.0.into();
+        let im = value.1.into();
         Self::C64(num::Complex::new(re, im))
     }
 }
@@ -725,86 +656,6 @@ impl FromStr for Complex {
     }
 }
 
-impl<'de> Deserialize<'de> for Complex {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        let [re, im]: [f64; 2] = Deserialize::deserialize(deserializer)?;
-        Ok(num::Complex::new(re, im).into())
-    }
-}
-
-impl Serialize for Complex {
-    fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
-        match self {
-            Complex::C32(c) => {
-                let mut seq = s.serialize_seq(Some(2))?;
-                seq.serialize_element(&c.re)?;
-                seq.serialize_element(&c.im)?;
-                seq.end()
-            }
-            Complex::C64(c) => {
-                let mut seq = s.serialize_seq(Some(2))?;
-                seq.serialize_element(&c.re)?;
-                seq.serialize_element(&c.im)?;
-                seq.end()
-            }
-        }
-    }
-}
-
-#[async_trait]
-impl FromStream for Complex {
-    type Context = ();
-
-    async fn from_stream<D: Decoder>(
-        cxt: Self::Context,
-        decoder: &mut D,
-    ) -> Result<Self, D::Error> {
-        let [re, im]: [f64; 2] = FromStream::from_stream(cxt, decoder).await?;
-        Ok(num::Complex::new(re, im).into())
-    }
-}
-
-impl<'en> ToStream<'en> for Complex {
-    fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
-        match self {
-            Complex::C32(c) => {
-                let mut seq = e.encode_seq(Some(2))?;
-                seq.encode_element(&c.re)?;
-                seq.encode_element(&c.im)?;
-                seq.end()
-            }
-            Complex::C64(c) => {
-                let mut seq = e.encode_seq(Some(2))?;
-                seq.encode_element(&c.re)?;
-                seq.encode_element(&c.im)?;
-                seq.end()
-            }
-        }
-    }
-}
-
-impl<'en> IntoStream<'en> for Complex {
-    fn into_stream<E: Encoder<'en>>(self, e: E) -> Result<E::Ok, E::Error> {
-        match self {
-            Complex::C32(c) => {
-                let mut seq = e.encode_seq(Some(2))?;
-                seq.encode_element(c.re)?;
-                seq.encode_element(c.im)?;
-                seq.end()
-            }
-            Complex::C64(c) => {
-                let mut seq = e.encode_seq(Some(2))?;
-                seq.encode_element(c.re)?;
-                seq.encode_element(c.im)?;
-                seq.end()
-            }
-        }
-    }
-}
-
 impl fmt::Debug for Complex {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{}: {}", self.class(), self)
@@ -1108,19 +959,6 @@ impl PartialOrd for Float {
     }
 }
 
-#[async_trait]
-impl async_hash::Hash for Float {
-    type Context = ();
-    type Error = Infallible;
-
-    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
-        match self {
-            Self::F32(f) => async_hash::Hash::hash(f, cxt).await,
-            Self::F64(f) => async_hash::Hash::hash(f, cxt).await,
-        }
-    }
-}
-
 impl Default for Float {
     fn default() -> Float {
         Float::F32(f32::default())
@@ -1219,54 +1057,6 @@ impl From<Float> for f64 {
     }
 }
 
-impl<'de> Deserialize<'de> for Float {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        f64::deserialize(deserializer).map(Self::from)
-    }
-}
-
-impl Serialize for Float {
-    fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
-        match self {
-            Float::F32(f) => s.serialize_f32(*f),
-            Float::F64(f) => s.serialize_f64(*f),
-        }
-    }
-}
-
-#[async_trait]
-impl FromStream for Float {
-    type Context = ();
-
-    async fn from_stream<D: Decoder>(
-        cxt: Self::Context,
-        decoder: &mut D,
-    ) -> Result<Self, D::Error> {
-        f64::from_stream(cxt, decoder).map_ok(Self::from).await
-    }
-}
-
-impl<'en> ToStream<'en> for Float {
-    fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
-        match *self {
-            Float::F32(f) => e.encode_f32(f),
-            Float::F64(f) => e.encode_f64(f),
-        }
-    }
-}
-
-impl<'en> IntoStream<'en> for Float {
-    fn into_stream<E: Encoder<'en>>(self, e: E) -> Result<E::Ok, E::Error> {
-        match self {
-            Float::F32(f) => e.encode_f32(f),
-            Float::F64(f) => e.encode_f64(f),
-        }
-    }
-}
-
 impl fmt::Debug for Float {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{}: {}", self.class(), self)
@@ -1690,21 +1480,6 @@ impl Ord for Int {
     }
 }
 
-#[async_trait]
-impl async_hash::Hash for Int {
-    type Context = ();
-    type Error = Infallible;
-
-    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
-        match self {
-            Self::I8(i) => async_hash::Hash::hash(i, cxt).await,
-            Self::I16(i) => async_hash::Hash::hash(i, cxt).await,
-            Self::I32(i) => async_hash::Hash::hash(i, cxt).await,
-            Self::I64(i) => async_hash::Hash::hash(i, cxt).await,
-        }
-    }
-}
-
 impl Default for Int {
     fn default() -> Int {
         Int::I16(i16::default())
@@ -1774,60 +1549,6 @@ impl FromStr for Int {
     }
 }
 
-impl<'de> Deserialize<'de> for Int {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        i64::deserialize(deserializer).map(Self::from)
-    }
-}
-
-impl Serialize for Int {
-    fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
-        match self {
-            Int::I8(i) => s.serialize_i8(*i),
-            Int::I16(i) => s.serialize_i16(*i),
-            Int::I32(i) => s.serialize_i32(*i),
-            Int::I64(i) => s.serialize_i64(*i),
-        }
-    }
-}
-
-#[async_trait]
-impl FromStream for Int {
-    type Context = ();
-
-    async fn from_stream<D: Decoder>(
-        cxt: Self::Context,
-        decoder: &mut D,
-    ) -> Result<Self, D::Error> {
-        i64::from_stream(cxt, decoder).map_ok(Self::from).await
-    }
-}
-
-impl<'en> ToStream<'en> for Int {
-    fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
-        match *self {
-            Int::I8(i) => e.encode_i8(i),
-            Int::I16(i) => e.encode_i16(i),
-            Int::I32(i) => e.encode_i32(i),
-            Int::I64(i) => e.encode_i64(i),
-        }
-    }
-}
-
-impl<'en> IntoStream<'en> for Int {
-    fn into_stream<E: Encoder<'en>>(self, e: E) -> Result<E::Ok, E::Error> {
-        match self {
-            Int::I8(i) => e.encode_i8(i),
-            Int::I16(i) => e.encode_i16(i),
-            Int::I32(i) => e.encode_i32(i),
-            Int::I64(i) => e.encode_i64(i),
-        }
-    }
-}
-
 impl fmt::Debug for Int {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{}: {}", self.class(), self)
@@ -2253,21 +1974,6 @@ impl Ord for UInt {
     }
 }
 
-#[async_trait]
-impl async_hash::Hash for UInt {
-    type Context = ();
-    type Error = Infallible;
-
-    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
-        match self {
-            Self::U8(u) => async_hash::Hash::hash(u, cxt).await,
-            Self::U16(u) => async_hash::Hash::hash(u, cxt).await,
-            Self::U32(u) => async_hash::Hash::hash(u, cxt).await,
-            Self::U64(u) => async_hash::Hash::hash(u, cxt).await,
-        }
-    }
-}
-
 impl Default for UInt {
     fn default() -> UInt {
         UInt::U8(u8::default())
@@ -2337,60 +2043,6 @@ impl FromStr for UInt {
     }
 }
 
-impl<'de> Deserialize<'de> for UInt {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        u64::deserialize(deserializer).map(Self::from)
-    }
-}
-
-impl Serialize for UInt {
-    fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
-        match self {
-            UInt::U8(u) => s.serialize_u8(*u),
-            UInt::U16(u) => s.serialize_u16(*u),
-            UInt::U32(u) => s.serialize_u32(*u),
-            UInt::U64(u) => s.serialize_u64(*u),
-        }
-    }
-}
-
-#[async_trait]
-impl FromStream for UInt {
-    type Context = ();
-
-    async fn from_stream<D: Decoder>(
-        cxt: Self::Context,
-        decoder: &mut D,
-    ) -> Result<Self, D::Error> {
-        u64::from_stream(cxt, decoder).map_ok(Self::from).await
-    }
-}
-
-impl<'en> ToStream<'en> for UInt {
-    fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
-        match *self {
-            UInt::U8(u) => e.encode_u8(u),
-            UInt::U16(u) => e.encode_u16(u),
-            UInt::U32(u) => e.encode_u32(u),
-            UInt::U64(u) => e.encode_u64(u),
-        }
-    }
-}
-
-impl<'en> IntoStream<'en> for UInt {
-    fn into_stream<E: Encoder<'en>>(self, e: E) -> Result<E::Ok, E::Error> {
-        match self {
-            UInt::U8(u) => e.encode_u8(u),
-            UInt::U16(u) => e.encode_u16(u),
-            UInt::U32(u) => e.encode_u32(u),
-            UInt::U64(u) => e.encode_u64(u),
-        }
-    }
-}
-
 impl fmt::Debug for UInt {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{}: {}", self.class(), self)
