@@ -2,43 +2,51 @@
 //! and [`UInt`] variants, as well as a [`NumberCollator`], [`ComplexCollator`], and
 //! [`FloatCollator`] since these types do not implement [`Ord`].
 //!
-//! `Number` supports casting with [`safecast`] and (de)serialization with [`serde`].
+//! `Number` supports casting with [`safecast`].
+//!
+//! For (de)serialization with `serde`, enable the `"serde"` feature.
+//!
+//! For (de)coding with `destream`, enable the `"stream"` feature.
 //!
 //! Example usage:
 //! ```
 //! # use number_general::{Int, Number};
 //! # use safecast::CastFrom;
-//! let sequence: Vec<Number> = serde_json::from_str("[true, 2, 3.5, -4, [1.0, -0.5]]").unwrap();
+//! let sequence: Vec<Number> = vec![true.into(), 2.into(), 3.5.into(), [1.0, -0.5].into()];
 //! let actual = sequence.into_iter().product();
-//! let expected = Number::from(num::Complex::<f64>::new(-28., 14.));
+//! let expected = Number::from(num::Complex::<f64>::new(7., -3.5));
 //!
 //! assert_eq!(expected, actual);
-//! assert_eq!(Int::cast_from(actual), Int::from(-28));
+//! assert_eq!(Int::cast_from(actual), Int::from(7));
 //! ```
 
 use std::cmp::Ordering;
-use std::convert::Infallible;
 use std::fmt;
 use std::iter::{Product, Sum};
 use std::ops::*;
 use std::str::FromStr;
 
-use async_trait::async_trait;
-use bytes::Bytes;
 use collate::*;
-use destream::de::{Decoder, Error as DestreamError, FromStream};
-use destream::en::{IntoStream, ToStream};
 use safecast::{CastFrom, CastInto};
-use serde::de::Error as SerdeError;
-use serde::ser::Serializer;
-use serde::{Deserialize, Deserializer, Serialize};
 
 mod class;
+#[cfg(feature = "stream")]
+mod destream;
+#[cfg(feature = "hash")]
+mod hash;
 mod instance;
+#[cfg(feature = "serde")]
+mod serde;
 
 pub use class::*;
 pub use instance::*;
 
+#[cfg(any(feature = "serde", feature = "stream"))]
+const ERR_COMPLEX: & str = "a complex number";
+
+#[cfg(any(feature = "serde", feature = "stream"))]
+const ERR_NUMBER: & str = "a Number, like 1 or -2 or 3.14 or [0., -1.414]";
+
 /// Define a [`NumberType`] for a non-[`Number`] type such as a Rust primitive.
 ///
 /// To access the `NumberType` of a `Number`, use `Instance::class`, e.g. `Number::from(1).class()`.
@@ -95,9 +103,6 @@ impl fmt::Display for Error {
 
 type _Complex<T> = num::complex::Complex<T>;
 
-const ERR_COMPLEX: &str = "a complex number";
-const ERR_NUMBER: &str = "a Number, like 1 or -2 or 3.14 or [0., -1.414]";
-
 /// A generic number.
 #[derive(Clone, Copy, Eq, Hash)]
 pub enum Number {
@@ -526,22 +531,6 @@ impl Trigonometry for Number {
     trig! {tanh}
 }
 
-#[async_trait]
-impl async_hash::Hash for Number {
-    type Context = ();
-    type Error = Infallible;
-
-    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
-        match self {
-            Self::Bool(b) => async_hash::Hash::hash(b, cxt).await,
-            Self::Complex(c) => async_hash::Hash::hash(c, cxt).await,
-            Self::Float(f) => async_hash::Hash::hash(f, cxt).await,
-            Self::Int(i) => async_hash::Hash::hash(i, cxt).await,
-            Self::UInt(u) => async_hash::Hash::hash(u, cxt).await,
-        }
-    }
-}
-
 impl Default for Number {
     fn default() -> Self {
         Self::Bool(Boolean::default())
@@ -638,6 +627,18 @@ impl From<Float> for Number {
     }
 }
 
+impl From<[f32; 2]> for Number {
+    fn from(arr: [f32; 2]) -> Self {
+        Self::Complex(arr.into())
+    }
+}
+
+impl From<[f64; 2]> for Number {
+    fn from(arr: [f64; 2]) -> Self {
+        Self::Complex(arr.into())
+    }
+}
+
 impl From<_Complex<f32>> for Number {
     fn from(c: _Complex<f32>) -> Self {
         Self::Complex(c.into())
@@ -838,8 +839,10 @@ impl Collate for NumberCollator {
 
 /// A struct for deserializing a `Number` which implements
 /// [`destream::de::Visitor`] and [`serde::de::Visitor`].
+#[cfg(any(feature = "serde", feature = "stream"))]
 pub struct NumberVisitor;
 
+#[cfg(any(feature = "serde", feature = "stream"))]
 impl NumberVisitor {
     #[inline]
     fn bool<E>(self, b: bool) -> Result<Number, E> {
@@ -897,229 +900,6 @@ impl NumberVisitor {
     }
 }
 
-impl<'de> serde::de::Visitor<'de> for NumberVisitor {
-    type Value = Number;
-
-    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.write_str(ERR_NUMBER)
-    }
-
-    #[inline]
-    fn visit_bool<E: SerdeError>(self, b: bool) -> Result<Self::Value, E> {
-        self.bool(b)
-    }
-
-    #[inline]
-    fn visit_i8<E: SerdeError>(self, i: i8) -> Result<Self::Value, E> {
-        self.i8(i)
-    }
-
-    #[inline]
-    fn visit_i16<E: SerdeError>(self, i: i16) -> Result<Self::Value, E> {
-        self.i16(i)
-    }
-
-    #[inline]
-    fn visit_i32<E: SerdeError>(self, i: i32) -> Result<Self::Value, E> {
-        Ok(Number::Int(Int::I32(i)))
-    }
-
-    #[inline]
-    fn visit_i64<E: SerdeError>(self, i: i64) -> Result<Self::Value, E> {
-        self.i64(i)
-    }
-
-    #[inline]
-    fn visit_u8<E: SerdeError>(self, u: u8) -> Result<Self::Value, E> {
-        self.u8(u)
-    }
-
-    #[inline]
-    fn visit_u16<E: SerdeError>(self, u: u16) -> Result<Self::Value, E> {
-        self.u16(u)
-    }
-
-    #[inline]
-    fn visit_u32<E: SerdeError>(self, u: u32) -> Result<Self::Value, E> {
-        self.u32(u)
-    }
-
-    #[inline]
-    fn visit_u64<E>(self, u: u64) -> Result<Self::Value, E> {
-        self.u64(u)
-    }
-
-    #[inline]
-    fn visit_f32<E>(self, f: f32) -> Result<Self::Value, E> {
-        self.f32(f)
-    }
-
-    #[inline]
-    fn visit_f64<E>(self, f: f64) -> Result<Self::Value, E> {
-        self.f64(f)
-    }
-
-    #[inline]
-    fn visit_str<E: SerdeError>(self, s: &str) -> Result<Self::Value, E> {
-        s.parse().map_err(serde::de::Error::custom)
-    }
-
-    #[inline]
-    fn visit_seq<A: serde::de::SeqAccess<'de>>(self, mut seq: A) -> Result<Self::Value, A::Error> {
-        let re = seq
-            .next_element()?
-            .ok_or_else(|| SerdeError::invalid_length(0, &ERR_COMPLEX))?;
-
-        let im = seq
-            .next_element()?
-            .ok_or_else(|| SerdeError::invalid_length(1, &ERR_COMPLEX))?;
-
-        Ok(Number::Complex(Complex::C64(_Complex::<f64>::new(re, im))))
-    }
-}
-
-#[async_trait]
-impl destream::de::Visitor for NumberVisitor {
-    type Value = Number;
-
-    fn expecting() -> &'static str {
-        ERR_NUMBER
-    }
-
-    #[inline]
-    fn visit_bool<E: DestreamError>(self, b: bool) -> Result<Self::Value, E> {
-        self.bool(b)
-    }
-
-    #[inline]
-    fn visit_i8<E: DestreamError>(self, i: i8) -> Result<Self::Value, E> {
-        self.i8(i)
-    }
-
-    #[inline]
-    fn visit_i16<E: DestreamError>(self, i: i16) -> Result<Self::Value, E> {
-        self.i16(i)
-    }
-
-    #[inline]
-    fn visit_i32<E: DestreamError>(self, i: i32) -> Result<Self::Value, E> {
-        self.i32(i)
-    }
-
-    #[inline]
-    fn visit_i64<E: DestreamError>(self, i: i64) -> Result<Self::Value, E> {
-        self.i64(i)
-    }
-
-    #[inline]
-    fn visit_u8<E: DestreamError>(self, u: u8) -> Result<Self::Value, E> {
-        self.u8(u)
-    }
-
-    #[inline]
-    fn visit_u16<E: DestreamError>(self, u: u16) -> Result<Self::Value, E> {
-        self.u16(u)
-    }
-
-    #[inline]
-    fn visit_u32<E: DestreamError>(self, u: u32) -> Result<Self::Value, E> {
-        self.u32(u)
-    }
-
-    #[inline]
-    fn visit_u64<E: DestreamError>(self, u: u64) -> Result<Self::Value, E> {
-        self.u64(u)
-    }
-
-    #[inline]
-    fn visit_f32<E: DestreamError>(self, f: f32) -> Result<Self::Value, E> {
-        self.f32(f)
-    }
-
-    #[inline]
-    fn visit_f64<E: DestreamError>(self, f: f64) -> Result<Self::Value, E> {
-        self.f64(f)
-    }
-
-    #[inline]
-    fn visit_string<E: DestreamError>(self, s: String) -> Result<Self::Value, E> {
-        s.parse().map_err(destream::de::Error::custom)
-    }
-
-    async fn visit_seq<A: destream::de::SeqAccess>(
-        self,
-        mut seq: A,
-    ) -> Result<Self::Value, A::Error> {
-        let re = seq
-            .next_element(())
-            .await?
-            .ok_or_else(|| DestreamError::invalid_length(0, ERR_COMPLEX))?;
-
-        let im = seq
-            .next_element(())
-            .await?
-            .ok_or_else(|| DestreamError::invalid_length(1, ERR_COMPLEX))?;
-
-        Ok(Number::Complex(Complex::C64(_Complex::<f64>::new(re, im))))
-    }
-}
-
-impl<'de> Deserialize<'de> for Number {
-    fn deserialize<D: Deserializer<'de>>(
-        deserializer: D,
-    ) -> Result<Self, <D as Deserializer<'de>>::Error> {
-        deserializer.deserialize_any(NumberVisitor)
-    }
-}
-
-#[async_trait]
-impl FromStream for Number {
-    type Context = ();
-
-    async fn from_stream<D: Decoder>(
-        _context: (),
-        decoder: &mut D,
-    ) -> Result<Self, <D as Decoder>::Error> {
-        decoder.decode_any(NumberVisitor).await
-    }
-}
-
-impl Serialize for Number {
-    fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
-        match self {
-            Number::Bool(b) => b.serialize(s),
-            Number::Complex(c) => c.serialize(s),
-            Number::Float(f) => f.serialize(s),
-            Number::Int(i) => i.serialize(s),
-            Number::UInt(u) => u.serialize(s),
-        }
-    }
-}
-
-impl<'en> ToStream<'en> for Number {
-    fn to_stream<E: destream::Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
-        match self {
-            Number::Bool(b) => b.to_stream(e),
-            Number::Complex(c) => c.to_stream(e),
-            Number::Float(f) => f.to_stream(e),
-            Number::Int(i) => i.to_stream(e),
-            Number::UInt(u) => u.to_stream(e),
-        }
-    }
-}
-
-impl<'en> IntoStream<'en> for Number {
-    fn into_stream<E: destream::Encoder<'en>>(self, e: E) -> Result<E::Ok, E::Error> {
-        match self {
-            Number::Bool(b) => b.into_stream(e),
-            Number::Complex(c) => c.into_stream(e),
-            Number::Float(f) => f.into_stream(e),
-            Number::Int(i) => i.into_stream(e),
-            Number::UInt(u) => u.into_stream(e),
-        }
-    }
-}
-
 impl fmt::Debug for Number {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{}: {}", self.class(), self)
@@ -1140,11 +920,6 @@ impl fmt::Display for Number {
 
 #[cfg(test)]
 mod tests {
-    use bytes::Bytes;
-    use futures::executor::block_on;
-    use futures::future::{self, FutureExt};
-    use futures::stream::{self, StreamExt};
-
     use super::*;
 
     #[test]
@@ -1216,58 +991,6 @@ mod tests {
         assert_eq!(collator.bisect_right(&numbers, &[Number::from(5.1)]), 5);
     }
 
-    #[test]
-    fn test_serialize() {
-        let numbers = [
-            Number::from(false),
-            Number::from(12u16),
-            Number::from(-3),
-            Number::from(3.14),
-            Number::from(_Complex::<f32>::new(0., -1.414)),
-        ];
-
-        for expected in &numbers {
-            let serialized = serde_json::to_string(expected).unwrap();
-            let actual = serde_json::from_str(&serialized).unwrap();
-
-            assert_eq!(expected, &actual);
-        }
-    }
-
-    #[test]
-    fn test_encode() {
-        let numbers = vec![
-            Number::from(false),
-            Number::from(12u16),
-            Number::from(-3),
-            Number::from(3.14),
-            Number::from(1e-6),
-            Number::from(_Complex::<f32>::new(0., -1.414)),
-        ];
-
-        let encoded = destream_json::encode(&numbers)
-            .unwrap()
-            .map(|r| r.unwrap())
-            .fold(vec![], |mut s, c| {
-                s.extend(c);
-                future::ready(s)
-            })
-            .map(Bytes::from);
-
-        let deserialized: Vec<Number> =
-            block_on(destream_json::decode((), stream::once(encoded))).unwrap();
-
-        assert_eq!(deserialized, numbers);
-
-        let fp: f64 = block_on(destream_json::decode(
-            (),
-            stream::once(future::ready(Bytes::copy_from_slice(b"1e-6"))),
-        ))
-        .unwrap();
-
-        assert_eq!(fp, 1e-6);
-    }
-
     #[test]
     fn test_parse() {
         assert_eq!(Number::from_str("12").unwrap(), Number::from(12));
